package istia.st.springmvc.exemples.web;

import java.io.DataOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.validation.BindException;
import org.springframework.web.bind.RequestUtils;
import org.springframework.web.bind.ServletRequestDataBinder;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.multipart.support.ByteArrayMultipartFileEditor;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;

public class UploadController extends SimpleFormController {

	// upload de document
	protected ModelAndView onSubmit(HttpServletRequest request,
			HttpServletResponse response, Object command, BindException errors)
			throws Exception {
		// on rcupre le document
		Document document = (Document) command;
		// on rcupre la requte multipart
		MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
		// on rcupre une rfrence sur le fichier charg
		MultipartFile fichier = multipartRequest.getFile("contenu");
		// fichier dsign ?
		if (fichier.isEmpty()) {
			// le fichier n'a pas t dsign
			errors.rejectValue("contenu", "document.contenu.obligatoire");
			// on raffiche le formulaire avec les erreurs
			return showForm(request, response, errors);
		}
		// on rcupre le nom du fichier charg
		String nomDocument = fichier.getOriginalFilename();
		// le chemin d'enregistrement du document
		String chemin = request.getSession().getServletContext().getRealPath(
				"/")
				+ "uploads\\" + nomDocument;
		// on rcupre le nom de l'action en cours
		String action = RequestUtils.getStringParameter(request, "action",
				"enregistrer1").toLowerCase();
		// selon l'action, deux mthodes diffrentes d'enregistrement
		if (action.equals("enregistrer1")) {
			doEnregistrer1(fichier, chemin);
		} else {
			doEnregistrer2(document, chemin);
		}
		// confirmation de l'enregistrement
		Map modle = new HashMap();
		modle.put("document", document);
		modle.put("nomDocument", nomDocument);
		return new ModelAndView("confirmation", modle);
	}

	// enregistrement - mthode 1 -  partir de la requte
	private void doEnregistrer1(MultipartFile fichier, String chemin)
			throws IllegalStateException, IOException {
		// on enregistre le flux [fichier] sur le serveur  l'emplacement
		// [chemin]
		fichier.transferTo(new File(chemin));
	}

	// enregistrement - mthode 2 -  partir du document
	private void doEnregistrer2(Document document, String chemin)
			throws IOException {
		// on enregistre le document [document] sur le serveur  l'emplacement
		// [chemin]
		DataOutputStream out = new DataOutputStream(
				new FileOutputStream(chemin));
		out.write(document.getContenu());
		out.close();
	}

	// enregistrement diteur de proprit
	protected void initBinder(HttpServletRequest request,
			ServletRequestDataBinder binder) throws Exception {
		// pour que le flux multipart post soit directement affect au champ de type byte[]
		binder.registerCustomEditor(byte[].class,
				new ByteArrayMultipartFileEditor());
	}

}
